package com.contentsquare.flutter.infrastructure.core.method_call_executor.agents

import com.contentsquare.flutter.infrastructure.core.method_call_executor.data.ExecutionData
import com.contentsquare.flutter.infrastructure.core.method_call_executor.responses.ExecutionResponse
import com.contentsquare.flutter.infrastructure.core.method_call_executor.responses.FailedExecutionResponse
import com.contentsquare.flutter.infrastructure.core.method_call_executor.responses.FailedExecutionResponseReason
import com.contentsquare.flutter.infrastructure.core.method_call_executor.responses.SuccessExecutionResponse
import com.contentsquare.flutter.infrastructure.proxies.SendSessionReplayEventProxy

/**
 * Allows to send a session replay event to Contentsquare Native SDK.
 */
internal class SendSessionReplayEventAgent(private val proxy: SendSessionReplayEventProxy = SendSessionReplayEventProxy.default()): MethodCallAgent(nativeExceptionMessage) {

    companion object {
        /** Message to use in case of error during native call. */
        const val nativeExceptionMessage = "Failed to send session replay event"

        /** Message in case of "body" parameter missing. */
        const val missingBodyMessage = "Missing string for the body parameter"

        /** Message in case of "pixelRatio" parameter missing. */
        const val missingPixelRatioMessage = "Missing float for the pixelRatio parameter"
    }

    override fun execute(data: ExecutionData): ExecutionResponse {
        val body = data.call.argument<String>("body")
            ?: return FailedExecutionResponse(FailedExecutionResponseReason.MISSING_ARGUMENT, missingBodyMessage, null)
        val pixelRatio = data.context?.resources?.displayMetrics?.density
                ?: return FailedExecutionResponse(FailedExecutionResponseReason.MISSING_ARGUMENT, missingPixelRatioMessage, null)

        proxy.send(body, pixelRatio)

        return SuccessExecutionResponse.empty()
    }

}