package com.contentsquare.flutter.infrastructure.proxies

import com.contentsquare.flutter.infrastructure.bridges.TelemetryInterfaceBridge

/**
 * Allows to send the flutter to Contentsquare API.
 */
internal interface CollectFlutterVersionProxy {
    /**
     * Collect the flutter version.
     *
     * @param version the flutter version
     */
    fun collect(version: String)

    companion object Factory {
        /**
         * Creates a new instance of the default implementation of [CollectFlutterVersionProxy].
         */
        fun default(telemetryInterfaceBridge: TelemetryInterfaceBridge = TelemetryInterfaceBridge.default()): CollectFlutterVersionProxy {
            return CollectFlutterVersionProxyImpl(telemetryInterfaceBridge)
        }
    }
}

/**
 * An implementation of [CollectFlutterVersionProxy].
 *
 * @see CollectFlutterVersionProxy
 */
internal class CollectFlutterVersionProxyImpl(private val telemetryInterfaceBridge: TelemetryInterfaceBridge): CollectFlutterVersionProxy {
    override fun collect(version: String) {
        telemetryInterfaceBridge.telemetrySetXPFVersion(version)
        telemetryInterfaceBridge.telemetrySetXPFType("flutter")
    }
}