/// Allows to collect custom telemetry the flutter to Contentsquare API.
protocol CollectFlutterVersionProxy {
    /// Collect the flutter version.
    ///
    /// - Parameter version: the flutter version to collect.
    func collect(version: String) throws -> Void
}

/// Factory creator class for [CollectFlutterVersionProxy].
struct CollectFlutterVersionProxyFactory {

    /// Creates a new instance of the default implementation of [CollectFlutterVersionProxy].
    static func makeDefault(telemetryInterfaceBridge: TelemetryInterfaceBridge = TelemetryInterfaceBridgeFactory.makeDefault()) -> CollectFlutterVersionProxy {
        return CollectFlutterVersionProxyImpl(telemetryInterfaceBridge: telemetryInterfaceBridge)
    }
}

/// An implementation of [CollectFlutterVersionProxy]
class CollectFlutterVersionProxyImpl: CollectFlutterVersionProxy {
    private let _telemetryInterfaceBridge: TelemetryInterfaceBridge

    init(telemetryInterfaceBridge: TelemetryInterfaceBridge) {
        _telemetryInterfaceBridge = telemetryInterfaceBridge
    }

    func collect(version: String) throws {
        try _telemetryInterfaceBridge.collectFlutterVersion(version: version)
    }
}
