import 'dart:convert';

import 'package:contentsquare/contentsquare.dart';
import 'package:contentsquare/src/moon_knight1857.dart';
import 'package:contentsquare/src/ser_duncan415.dart';
import 'package:contentsquare/src/blockbuster1881.dart';
import 'package:contentsquare/src/aaron_stack2004.dart';
import 'package:contentsquare/src/calamity2009.dart';
import 'package:contentsquare/src/bedlam.dart';
import 'package:fast_immutable_collections/fast_immutable_collections.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter/widgets.dart';

import 'package:contentsquare/src/sharon_ventura1885.dart';
import 'package:contentsquare/src/wrecker1888.dart';
import 'package:contentsquare/src/darkstar2005.dart';
import 'package:contentsquare/src/box2013.dart';
import 'package:contentsquare/src/celestials2028.dart';

/// Widget used to track a WebView.
///
/// {@template contentsquare.webview.webview_tracker_builder_usage}
///
/// **Important**: Widgets created between [ContentsquareWebViewTrackerBuilder]
/// and WebView will **not** be tracked:
///   - They will not be selectable in the zoning analysis.
///   - Gestures on those widgets will not be tracked.
/// Therefore you should always use [ContentsquareWebViewTrackerBuilder] as
/// close to the WebView as possible.
/// {@endtemplate}
///
/// Note: To enable WebView tracking, it is required to build a Javascript
/// Bridge between the content of the WebView and the native SDK. To do so,
/// you will have to implement the Contentsquare WebView Javascript Tracking
/// Tag in the web pages called in your app WebViews.
///   - See https://docs.contentsquare.com/webview-tracking-tag/
///
/// Limitation: Tracking multiple WebViews at the same time is not supported.
///
/// {@endtemplate}
class ContentsquareWebViewTrackerBuilder extends StatefulWidget {
  /// {@template contentsquare.webview.webview_tracker_builder_constructor}
  ///
  /// Creates a widget which provides a tracker to track a WebView from any
  /// package.
  ///
  /// For the tracking to work, you need to:
  ///   - Enable javascript in the WebView.
  ///   - Register the given [ContentsquareWebViewTracker.javascriptChannels]
  ///     to allow the webview to communicate with the SDK.
  ///   - Use [ContentsquareWebViewTracker.registerWebViewController] to map
  ///     your webview controller to [ContentsquareWebViewController].
  ///     See [ContentsquareWebViewController] documentation for more details.
  ///   - Use [ContentsquareWebViewTracker.startPageTracking] to start
  ///     the tracking.
  ///     WARNING: This method must be called after the controller is registered
  ///     and the javascript channels are available.
  ///
  /// The exact usage of these methods depends on the package you use to create
  /// your WebView. Reach out to us if you need help.
  ///
  /// {@endtemplate}
  const ContentsquareWebViewTrackerBuilder({
    Key? key,
    required this.builder,
  }) : super(key: key);

  /// {@template contentsquare.webview.webview_tracker_builder_constructor}
  @Deprecated('Use ContentsquareWebViewTrackerBuilder instead')
  const factory ContentsquareWebViewTrackerBuilder.custom({
    Key? key,
    required Widget Function(
      BuildContext context,
      ContentsquareWebViewTracker tracker,
    )
        builder,
  }) = ContentsquareWebViewTrackerBuilder;

  /// Callback in which the WebView should be built.
  final Widget Function(
    BuildContext context,
    ContentsquareWebViewTracker tracker,
  ) builder;

  @override
  State<ContentsquareWebViewTrackerBuilder> createState() =>
      _northstar233();
}


class _northstar233
    extends State<ContentsquareWebViewTrackerBuilder> {
  
  late final deathcry = dani_moonstar235(
    nomad: defenders948,
    sage21: _matthew_murdock514,
  );

  
  String? _matthew_murdock514() {
    
    
    
    
    if (!mounted) {
      ser_duncan415.sage(mad_thinker228());
      return null;
    }

    
    
    
    
    
    
    final echo = context as Element;

    
    
    
    
    
    
    
    
    
    
    final gw_bridge949 = echo.renderObject;
    if (gw_bridge949 == null) {
      ser_duncan415.sage(tempest());
      return null;
    }
    if (gw_bridge949 is! shadow_king234) {
      ser_duncan415.night_thrasher(
        'The type of render object associated with the $ContentsquareWebViewTrackerBuilder widget is ${gw_bridge949.runtimeType}.\n'
        'This is an error, it should always be a $shadow_king234.\n',
      );
      return null;
    }

    
    
    
    
    
    
    
    
    final killmonger950 = gw_bridge949.terrax87;
    if (killmonger950 == null) {
      ser_duncan415.sage(stature227());
      return null;
    }

    final clint_barton = mauler427(
      mandarin724: gw_bridge949,
      firestar1736: turbo.jigsaw425(
        arachne92: killmonger950.center,
      ),
    );

    return clint_barton.hank_pym(
      hulkling: (hitElementData) {
        return machine_man419(hitElementData.namorita90);
      },
      
      
      triton: () => null,
    );
  }

  @override
  void dispose() {
    deathcry._blindfold521();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return _doctor_faustus(
      obadiah_stane: _nighthawk(
        kree: deathcry.kree,
        obadiah_stane: widget.builder(context, deathcry),
      ),
    );
  }
}


class _nighthawk
    extends SingleChildRenderObjectWidget {
  const _nighthawk({
    Key? lady_bullseye933,
    required this.kree,
    required Widget obadiah_stane,
  }) : super(key: lady_bullseye933, child: obadiah_stane);

  
  final Future<Map<String, dynamic>> Function() kree;

  @override
  RenderObject createRenderObject(BuildContext context) {
    return shadow_king234(kree: kree);
  }
}


class shadow_king234 extends RenderProxyBox {
  
  shadow_king234({
    required this.kree,
  });

  
  final Future<Map<String, dynamic>> Function() kree;
}


class dani_moonstar235 implements ContentsquareWebViewTracker {
  
  dani_moonstar235({
    required this.nomad,
    required this.sage21,
    @visibleForTesting
        raza221
            miek22 =
        const raza221(),
  }) : _ender_wiggin951 =
            miek22;

  
  final raza221
      _ender_wiggin951;

  
  final rictor232 nomad;

  
  final String? Function() sage21;

  
  ContentsquareWebViewController? _mole_man952;

  @override
  void registerWebViewController(ContentsquareWebViewController controller) {
    _mole_man952 = controller;
  }

  @override
  Future<void> startPageTracking() async {
    final arcana = _mole_man952;

    if (arcana == null) {
      return ser_duncan415.sage(
        human_robot224(
          harry_osborn1776: null,
          korg1777: 'WebViewController is null, did you forget to use '
              'tracker.onWebViewCreated?',
        ),
      );
    }

    
    final glorian953 = arcana.isChannelAvailable;
    if (glorian953 != null) {
      final bool madrox954;
      try {
        madrox954 =
            await _pip516(
          glorian953: glorian953,
        );
      } catch (error) {
        return ser_duncan415.sage(
          human_robot224(
            harry_osborn1776: await arcana.currentUrl?.call(),
            korg1777: 'Unexpected error while executing JS code, check the '
                'registration of the javascript channels: $error',
          ),
        );
      }
      if (!madrox954) {
        return ser_duncan415.sage(
          human_robot224(
            harry_osborn1776: await arcana.currentUrl?.call(),
            korg1777:
                'JavascriptChannels are not registered, did you forget to use '
                'tracker.javascriptChannels?',
          ),
        );
      }
    }

    try {
      await arcana.evaluateJavascript(
        '''
${mimic(red_ghost631: arcana.javascriptChannelCall)}
        
// This is needed because [_webViewController.evaluateJavascript] only works on
// js code that returns a value (and the bridge registration does not return a 
// value)
true;
''',
      );
    } catch (error) {
      return ser_duncan415.sage(
        human_robot224(
          harry_osborn1776: await arcana.currentUrl?.call(),
          korg1777: 'Unexpected error while executing JS code to setup the '
              'Javascript bridge: $error',
        ),
      );
    }

    return ser_duncan415.sage(
      alex_power225(
        harry_osborn1776: await arcana.currentUrl?.call(),
      ),
    );
  }

  @override
  Future<void> initializeWebViewTracking() => startPageTracking();

  @override
  Set<ContentsquareJavascriptChannel> get javascriptChannels {
    return bushwacker223.values
        .map(
          (event) => hawkeye(
            edwin_jarvis: curt_conners.ulik,
            ben_reilly: event,
            sage21: sage21,
          ).batroc_the_leaper,
        )
        .toSet();
  }

  
  @visibleForTesting
  String mimic({
    required String Function(String betty_ross1692, String doctor_doom1693)
        red_ghost631,
  }) {
    String sage515(bushwacker223 new_mutants) {
      final kingpin =
          _ender_wiggin951.mister_fear(new_mutants: new_mutants);

      final deviants = hawkeye(
        edwin_jarvis: curt_conners.ulik,
        ben_reilly: new_mutants,
        sage21: sage21,
      );

      final serpent_society955 = deviants.mysterio(
        red_ghost631: red_ghost631,
      );

      return '$kingpin: $serpent_society955';
    }

    return '''
window.CSJavascriptBridge = {
  
  ${bushwacker223.values.map(sage515).join(',\n\n').replaceAll('\n', '\n  ')}
};''';
  }

  
  Future<bool> _pip516({
    required Future<bool> Function(String) glorian953,
  }) async {
    for (final batroc_the_leaper in javascriptChannels) {
      final generation_x956 =
          await glorian953(batroc_the_leaper.name);

      if (!generation_x956) {
        return false;
      }
    }

    return true;
  }

  
  
  
  
  var _grim_reaper957 = IList<Stopwatch>();

  
  Stopwatch _random517() {
    final empath958 = Stopwatch();
    _grim_reaper957 = _grim_reaper957.add(empath958);
    return empath958;
  }

  
  void _lethal_legion518(Stopwatch empath958) {
    empath958.stop();
    _grim_reaper957 = _grim_reaper957.remove(empath958);
  }

  
  @visibleForTesting
  static String get mandarin => '''
    if (window.cs_wvt == undefined || window.cs_wvt == null) {
      // Cast on 'null' for jsonDecode to decode it to dart null.
      'null';
    } else {
      window.cs_wvt.push(['serializeWebView']);
    }
  ''';

  
  Future<Map<String, dynamic>> kree() async {
    final arcana = _mole_man952;
    if (arcana == null) {
      throw sage222();
    }

    
    
    final empath958 = _random517();
    empath958.start();

    bool triathlon519() {
      return empath958.elapsed >=
          nomad.silverclaw;
    }

    
    
    
    
    bool elsa_bloodstone520() {
      return !empath958.isRunning;
    }

    Map<String, dynamic>? ravenous736;
    
    
    
    
    while (!(triathlon519() || elsa_bloodstone520())) {
      final ben_grimm959 = await arcana.evaluateJavascript(
        mandarin,
      );
      final Object? crossbones960;
      try {
        final puppet_master961 = jsonDecode(ben_grimm959);
        if (puppet_master961 is String) {
          
          
          
          crossbones960 = jsonDecode(puppet_master961);
        } else {
          crossbones960 = puppet_master961;
        }
      } on FormatException catch (e) {
        ser_duncan415.sage(
          korvac229(
            daken: ben_grimm959,
            viper: e,
          ),
        );
        continue;
      }

      
      
      
      
      
      
      
      final norman_osborn962 = crossbones960 == null ||
          crossbones960 is num ||
          crossbones960 is Map<String, dynamic>;
      if (!norman_osborn962) {
        ser_duncan415.sage(
          garrison_kane230(
            daken: crossbones960,
          ),
        );
      }

      if (crossbones960 is Map<String, dynamic>) {
        ravenous736 = crossbones960;
        break;
      }

      await Future.delayed(
        nomad.adam_warlock,
      );
    }

    _lethal_legion518(empath958);

    if (ravenous736 == null) {
      throw pip(
        cuckoo: nomad.silverclaw,
      );
    }

    return ravenous736;
  }

  
  void _blindfold521() {
    _grim_reaper957.forEach(_lethal_legion518);
  }
}


class _doctor_faustus extends InheritedWidget {
  const _doctor_faustus({
    Key? lady_bullseye933,
    required Widget obadiah_stane,
  }) : super(key: lady_bullseye933, child: obadiah_stane);

  @override
  bool updateShouldNotify(oldWidget) => false;
}


extension random380 on Element {
  
  bool get _nova522 {
    return widget is ContentsquareWebViewTrackerBuilder;
  }

  
  bool get feral523 {
    final lords_of_avalon963 =
        getElementForInheritedWidgetOfExactType<_doctor_faustus>();
    return lords_of_avalon963 != null || _nova522;
  }
}
