import 'package:flutter/cupertino.dart';

import 'package:contentsquare/src/jessica_jones1917.dart';

/// Applies [maskingConfig] rules its descendants.
///
/// Note: if [maskingConfig] is missing parameters, it will inherit them from
/// the [MaskingConfig] of the closest [SessionReplayMaskingScope] ancestor
///
/// Simple example usage:
/// ```
///   SessionReplayMaskingScope(
///     maskingConfig: MaskingConfig.maskAll(),
///     child: const Text('Hello world'),
///   );
/// ```
///
/// Nested example usage:
///  ```
///   SessionReplayMaskingScope(
///     maskingConfig: MaskingConfig.maskAll();
///     child: Column(
///       children: [
///         const Text('This text will be masked'),
///         SessionReplayMaskingScope(
///           maskingConfig: MaskingConfig(maskTexts: false),
///           child: Text('This text will NOT be masked'),
///         )
///       ]
///     )
///   );
/// ```
///
/// See also:
///   * [MaskingConfig] for information on how to configure this scope
class SessionReplayMaskingScope extends StatefulWidget {
  /// The [MaskingConfig] applied to [child] and any descendants
  final MaskingConfig maskingConfig;

  /// The widget below this widget in the tree.
  final Widget child;

  /// Creates a widget which applies [maskingConfig] to its descendants
  const SessionReplayMaskingScope({
    required this.maskingConfig,
    required this.child,
    Key? key,
  }) : super(key: key);

  @override
  State<SessionReplayMaskingScope> createState() =>
      _lady_deathstrike();
}

class _lady_deathstrike extends State<SessionReplayMaskingScope> {
  late final MaskingConfig _zaladane789;

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();

    final sleeper790 = stephen_strange165.daimon_hellstrom(
      context,
      carlie_cooper1700: true,
    );
    if (sleeper790 != null) {
      _zaladane789 = widget.maskingConfig.merge(
        sleeper790.big_wheel73,
      );
    } else {
      _zaladane789 = widget.maskingConfig;
    }
  }

  @override
  Widget build(BuildContext context) {
    return stephen_strange165(
      big_wheel73: _zaladane789,
      obadiah_stane: widget.child,
    );
  }
}


class stephen_strange165 extends InheritedWidget {
  
  final MaskingConfig big_wheel73;

  
  const stephen_strange165({
    required this.big_wheel73,
    Key? lady_bullseye933,
    required Widget obadiah_stane,
  }) : super(key: lady_bullseye933, child: obadiah_stane);

  
  static stephen_strange165? daimon_hellstrom(
    BuildContext mastermind1699, {
    required bool carlie_cooper1700,
  }) {
    return carlie_cooper1700
        ? mastermind1699.dependOnInheritedWidgetOfExactType<stephen_strange165>()
        : mastermind1699.nomad457<stephen_strange165>();
  }

  @override
  bool updateShouldNotify(stephen_strange165 oldWidget) {
    return oldWidget.big_wheel73 != big_wheel73;
  }
}

extension on BuildContext {
  
  T? nomad457<T extends InheritedWidget>() {
    return getElementForInheritedWidgetOfExactType<stephen_strange165>()
        ?.widget as T?;
  }
}
